@@ -33,12 +33,13 @@
 #' @docType class
 #' @name dfm-class
 setClass("dfm",
-         slots = c(settings = "list", weighting = "character", smooth = "numeric",
+         slots = c(settings = "list", weightTf = "list", weightDf = "list", smooth = "numeric",
                    ngrams = "integer", concatenator = "character"),
          prototype = list(settings = list(NULL),
                           Dim = integer(2), 
                           Dimnames = list(docs=NULL, features=NULL),
-                          weighting = "frequency", 
+                          weightTf = list(scheme = "count", base = NULL, K = NULL),
+                          weightDf = list(scheme = "unary", base = NULL, c = NULL, smoothing = NULL, threshold = NULL),
                           smooth = 0,
                           ngrams = 1L,
                           concatenator = ""),
@@ -114,11 +115,20 @@ head.dfm <- function(x, n = 6L, nfeature = 6L, ...) {
     return(invisible(x[1:min(ndoc(x), n), 1:min(nfeature(x), nfeature)]))
 }
 
+#' @export
+#' @rdname head.dfm
+setMethod("head", signature(x = "dfm"), function(x, n = 6L, nfeature = 6L, ...) 
+    head.dfm(x, n = n, nfeature = nfeature, ...))
+
+#' @export
+#' @rdname head.dfm
+setMethod("tail", signature(x = "dfm"), function(x, n = 6L, nfeature = 6L, ...) 
+    tail.dfm(x, n = n, nfeature = nfeature, ...))
+
+
 #' @export
 #' @method tail dfm
 #' @rdname head.dfm
-#' @return A \link{dfm-class} class object corresponding to the subset
-#'   defined by \code{n} and \code{ncol}.
 tail.dfm <- function(x, n = 6L, nfeature = 6L, ...) {
     if (length(addedArgs <- list(...)))
         warning("Argument", ifelse(length(addedArgs)>1, "s ", " "), names(addedArgs), " not used.", sep = "")
@@ -238,13 +248,13 @@ print.dfm <- function(x, show.values=FALSE, show.settings=FALSE, show.summary =
 # @method t dfmSparse
 #setMethod("t", signature(x = "dfmSparse"), getMethod("t", "dgCMatrix"))
 
-## S4 Method for the S4 class dense/weighted dfm
-# @rdname dfm-class
-# @method t dfmSparse
-# setMethod("t", signature(x = "dfmDense"), definition = 
-#               function(x) {
-#                   selectMethod("t", "dgeMatrix")
-#               }) #getMethod("t", "dgeMatrix"))
+# S4 Method for the S4 class dense/weighted dfm
+#' @export
+#' @rdname dfm-class
+setMethod("t", signature(x = "dfmDense"), definition = 
+              function(x) {
+                  getMethod("t", "dgeMatrix")(x)
+              }) #getMethod("t", "dgeMatrix"))
 
 
 ## S4 Method for the S3 class dense dfm
@@ -252,44 +262,12 @@ print.dfm <- function(x, show.values=FALSE, show.settings=FALSE, show.summary =
 #' @param x the dfm object
 #' @rdname dfm-class 
 setMethod("t",
-          signature = (x = "dfm"),
+          signature = (x = "dfmSparse"),
           definition = function(x) {
-              newx <- t(matrix(x, nrow=nrow(x)))
-              dimnames(newx) <- rev(dimnames(x))
-#               if (isS4(x)) {
-#                   newx <- t(as.Matrix(x))
-#                   attributes(newx)$dimnames <- rev(x@Dimnames)
-#               } else {
-#                   attsorig <- attributes(x)
-#                   attributes(newx)$dimnames <- rev(attsorig$dimnames)
-#               }
-              newx
+              new("dfmSparse", getMethod("t", "dgCMatrix")(x))
           })
 
 
-# @details \code{rowSums} and \code{colSums} form row and column sums and means for \link{dfm-class} objects.
-# @param x a dfm, inheriting from \link[Matrix]{Matrix}
-# @param na.rm if \code{TRUE}, omit missing values (including \code{NaN}) from
-#   the calculations
-# @param dims ignored
-# @param ... additional arguments, for methods/generic compatibility
-# @return returns a named (non-sparse) numeric vector
-# @rdname dfm-class
-# @aliases colSums rowSums
-# @export
-# @examples
-# myDfm <- dfm(inaugTexts, verbose=FALSE)
-# colSums(myDfm[, 1:10])
-# rowSums(myDfm)
-# @export
-# setGeneric("colSums",
-#           def = function(x, na.rm = FALSE, dims = 1L, ...) standardGeneric("colSums"))
-# 
-# # @export
-# # @rdname dfm-class
-# setGeneric("rowSums",
-#            def = function(x, na.rm = FALSE, dims = 1L, ...) standardGeneric("rowSums"))
-
 
 #' @method colSums dfmSparse
 #' @rdname dfm-class
@@ -299,59 +277,39 @@ setMethod("t",
 #' @export
 setMethod("colSums", 
           signature = (x = "dfmSparse"),
-          definition = function(x, na.rm = FALSE, dims = 1L, ...) {
-              csums <- callNextMethod()
-              names(csums) <- features(x)
-              csums
-          })
-
-#' @method colSums dfmDense
-#' @rdname dfm-class
-#' @export
-setMethod("colSums", 
-          signature = (x = "dfmDense"),
-          definition = function(x, na.rm = FALSE, dims = 1L, ...) {
-              csums <- callNextMethod()
-              names(csums) <- features(x)
-              csums
-          })
+          function(x, na.rm = FALSE, dims = 1L, ...) callNextMethod())
 
 #' @method rowSums dfmSparse
 #' @rdname dfm-class
 #' @export
 setMethod("rowSums", 
           signature = (x = "dfmSparse"),
-          definition = function(x, na.rm = FALSE, dims = 1L, ...) {
-              rsums <- callNextMethod()
-              names(rsums) <- docnames(x)
-              rsums
-          })
-
-#' @method rowSums dfmDense
-#' @rdname dfm-class
-#' @export
-setMethod("rowSums", 
-          signature = (x = "dfmDense"),
-          definition = function(x, na.rm = FALSE, dims = 1L, ...) {
-              rsums <- callNextMethod()
-              names(rsums) <- docnames(x)
-              rsums
-          })
+          function(x, na.rm = FALSE, dims = 1L, ...) callNextMethod())
 
 
 
-## S3 METHODS FOR INDEXING DENSE dfm object
-#' @export
-#' @method [ dfm
-#' @rdname dfm-class
-`[.dfm` <- function(x, i, j, ..., drop=FALSE) {
-    if (drop) warning("drop=TRUE not currently supported")
-    m <- NextMethod("[", drop=FALSE)
-    attr(m, "settings") <- attr(x, "settings")
-    attr(m, "weighting") <- attr(x, "weighting")
-    class(m) <- class(x)
-    m
-}
+# #' @param i index for documents
+# #' @param j index for features
+# #' @param drop always set to \code{FALSE}
+# #' @param ... additional arguments not used here
+# #' @export
+# #' @rdname dfm-class
+# setMethod("[", 
+#           signature = c(x = "dfmSparse", i = "ANY", j = "ANY", drop = "logical"),
+#           function(x, i, j, ..., drop = FALSE) {
+#               cat("XXXXXX\n")
+#               new("dfmSparse", getMethod("[", "dgCMatrix")(x, i, j, ..., drop = FALSE))
+#               })
+# 
+# #' @rdname dfm-class
+# #' @export
+# setMethod("[", 
+#           signature = (x = "dfmDense"),
+#           function(x, i, j, ..., drop = FALSE) {
+#               cat("OH YEAH\n")
+#               new("dfmDense", callNextMethod())
+#           })
+# 
 
 ## S4 METHODS FOR INDEXING SPARSE dfm (dfmSparse) objects
 
