@@ -45,26 +45,25 @@ setMethod("show",
 #' a set of texts in columns and document variables and document-level meta-data
 #' in additional columns.  For spreadsheet-like files, the first row must be a 
 #' header.
-#' @param file the complete filename to be read.  Currently available file types
-#'   are: \describe{ \item{\code{txt}}{plain text files} \item{\code{json}}{data
-#'   in JavaScript Object Notation, consisting of the texts and additional 
-#'   document-level variables and document-level meta-data.  The text key must 
-#'   be identified by specifying a \code{textField} value.} 
-#'   \item{\code{csv}}{comma separated value data, consisting of the texts and 
-#'   additional document-level variables and document-level meta-data.  The text
-#'   file must be identified by specifying a \code{textField} value.} 
-#'   \item{\code{tab, tsv}}{tab-separated value data, consisting of the texts and 
-#'   additional document-level variables and document-level meta-data.  The text
-#'   file must be identified by specifying a \code{textField} value.} 
-#'   \item{a 
-#'   wildcard value}{any valid pathname with a wildcard ("glob") expression that
-#'   can be expanded by the operating system.  This may consist of multiple file
-#'   types.} \item{\code{xml}}{Basic flat XML documents are supported -- those 
-#'   of the kind supported by the function xmlToDataFrame function of the 
-#'   \strong{XML}  package.} 
-#'   \item{\code{zip}}{zip archive file, containing \code{*.txt} files.  This may be 
-#'   a URL to a zip file.}
-#'   }
+#' @param file the complete filename(s) to be read.  The value can be a vector
+#'   of file names, a single file name, or a file "mask" using a "glob"-type
+#'   wildcard value.  Currently available file value types are: \describe{
+#'   \item{\code{txt}}{plain text files} \item{\code{json}}{data in JavaScript
+#'   Object Notation, consisting of the texts and additional document-level
+#'   variables and document-level meta-data.  The text key must be identified by
+#'   specifying a \code{textField} value.} \item{\code{csv}}{comma separated
+#'   value data, consisting of the texts and additional document-level variables
+#'   and document-level meta-data.  The text file must be identified by
+#'   specifying a \code{textField} value.} \item{\code{tab, tsv}}{tab-separated
+#'   value data, consisting of the texts and additional document-level variables
+#'   and document-level meta-data.  The text file must be identified by
+#'   specifying a \code{textField} value.} \item{a wildcard value}{any valid
+#'   pathname with a wildcard ("glob") expression that can be expanded by the
+#'   operating system.  This may consist of multiple file types.}
+#'   \item{\code{xml}}{Basic flat XML documents are supported -- those of the
+#'   kind supported by the function xmlToDataFrame function of the \strong{XML} 
+#'   package.} \item{\code{zip}}{zip archive file, containing \code{*.txt}
+#'   files.  This may be a URL to a zip file.} }
 #' @param textField a variable (column) name or column number indicating where 
 #'   to find the texts that form the documents for the corpus.  This must be 
 #'   specified for file types \code{.csv} and \code{.json}.
@@ -82,8 +81,8 @@ setMethod("show",
 #' @param docvarsfrom  used to specify that docvars should be taken from the 
 #'   filenames, when the \code{textfile} inputs are filenames and the elements 
 #'   of the filenames are document variables, separated by a delimiter 
-#'   (\code{dvsep}).  This allows easy assignment of docvars from filenames such 
-#'   as \code{1789-Washington.txt}, \code{1793-Washington}, etc. by \code{dvsep} 
+#'   (\code{dvsep}).  This allows easy assignment of docvars from filenames such
+#'   as \code{1789-Washington.txt}, \code{1793-Washington}, etc. by \code{dvsep}
 #'   or from meta-data embedded in the text file header (\code{headers}).
 #' @param dvsep separator used in filenames to delimit docvar elements if 
 #'   \code{docvarsfrom="filenames"} is used
@@ -148,6 +147,10 @@ setGeneric("textfile",
 #'           file = "/tmp/inaugTexts.csv", row.names = FALSE)
 #' mytf7 <- textfile("/tmp/inaugTexts.csv", textField = "inaugSpeech")
 #' summary(corpus(mytf7))
+#' 
+#' # vector of full filenames for a recursive structure
+#' textfile(list.files(path = "~/Desktop/texts", pattern = "\\.txt$", 
+#'                     full.names = TRUE, recursive = TRUE))
 #' }
 setMethod("textfile", 
           signature(file = "character", textField = "index", 
@@ -177,12 +180,12 @@ setMethod("textfile",
                     encodingFrom="ANY", encodingTo="ANY", cache = "ANY",
                     docvarsfrom="missing", dvsep="missing", docvarnames="missing"),
           definition = function(file, encodingFrom = NULL, encodingTo = "UTF-8", cache = FALSE, ...) {
-              if (length(addedArgs <- list(...)))
-                  warning("Argument", ifelse(length(addedArgs)>1, "s ", " "), names(addedArgs), " not used.", sep = "")
+#               if (length(addedArgs <- list(...)))
+#                   warning("Argument", ifelse(length(addedArgs)>1, "s ", " "), names(addedArgs), " not used.", sep = "")
               
               fileType <- getFileType(file)
-              if (fileType=="filemask") {
-                  sources <- get_docs(file, encodingFrom, encodingTo)
+              if (fileType=="filemask" | fileType=="vector") {
+                  sources <- get_docs(file, encodingFrom, encodingTo, ...)
               } else {
                   sources <- get_doc(file, encodingFrom, encodingTo)
               }
@@ -236,12 +239,10 @@ get_doc <- function(f, encodingFrom = NULL, encodingTo = "UTF-8") {
     switch(fileType,
            txt =  { 
                if (is.null(encodingFrom)) encodingFrom <- getOption("encoding")
-               fn <- file(f, open = "rt", encoding = encodingFrom)
-               result <- list(txts = paste(suppressWarnings(readLines(fn)), collapse="\n"), docv=data.frame())
-               close(fn)
-               if (encodingTo != "UTF-8")
-                   result <- iconv(result, from = "UTF-8", to = encodingTo)
-               return(result)
+               txts <- readLines(f, encoding = encodingFrom, warn = FALSE)
+               if (!(encodingTo %in% c("UTF-8", "utf-8", "UTF8")))
+                   txts <- iconv(txts, from = encodingFrom, to = encodingTo, sub = "")
+               return(list(txts = paste(txts, collapse="\n"), docv=data.frame()))
            },
            doc =  { return(list(txts = get_word(f), docv=data.frame())) },
            json = { return(get_json_tweets(f)) },
@@ -251,14 +252,20 @@ get_doc <- function(f, encodingFrom = NULL, encodingTo = "UTF-8") {
     stop("unrecognized fileType:", fileType)
 }
 
-get_docs <- function(filemask, encodingFrom = NULL, encodingTo = "UTF-8") {
-    # get the pattern at the end
-    pattern <- getRootFileNames(filemask)
-    # get the directory name
-    path <- substr(filemask, 1, nchar(filemask) - nchar(pattern))
-    if (path == "") path <- "."
-    # get the filenames
-    filenames <- list.files(path, pattern, full.names=TRUE)
+get_docs <- function(filemask, encodingFrom = NULL, encodingTo = "UTF-8", ...) {
+    
+    if (length(filemask) == 1) {
+        # get the pattern at the end
+        pattern <- getRootFileNames(filemask)
+        # get the directory name
+        path <- substr(filemask, 1, nchar(filemask) - nchar(pattern))
+        if (path == "") path <- "."
+        # get the filenames
+        filenames <- list.files(path, pattern, full.names=TRUE)
+    } else {
+        filenames <- filemask
+    }
+    
     # read texts from call to get_doc, discarding any docv
     if (!is.null(encodingFrom)) {
         if (length(encodingFrom) > 1) {
@@ -408,6 +415,8 @@ get_xml <- function(file, textField, sep=",", ...) {
 
 
 getFileType <- function(filenameChar) {
+    if (length(filenameChar) > 1)
+        return("vector")
     if (!substr(filenameChar, 1, 4)=="http" & grepl("[?*]", filenameChar))
         return("filemask")
     filenameParts <- strsplit(filenameChar, ".", fixed=TRUE)
